//    This program is free software; you can redistribute it and/or modify
//    it under the terms of the GNU General Public License as published by
//    the Free Software Foundation; either version 2 of the License, or
//    (at your option) any later version.
//
//    This program is distributed in the hope that it will be useful,
//    but WITHOUT ANY WARRANTY; without even the implied warranty of
//    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
//    GNU General Public License for more details.
//
//    You should have received a copy of the GNU General Public License
//    along with this program; if not, write to the Free Software
//    Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.

//
//	this file was originally created by Olaf Hartig (9 Nov 1999)
//

#include "CConfigDialog.h"
#include <qpushbt.h>
#include <qlineedit.h>
#include <qkeycode.h>

// added by Holger Priebs ( 18 Nov 99 )
#include "configback.xpm"
#include "nw.xpm"
#include "sw.xpm"
#include "ne.xpm"
#include "se.xpm"
// -----------------------------------

// declaration of non-member-function
QString getKeycodeName(int nKeycode);

// defining control's sizes
#define LINEEDIT_HEIGHT	25 // the height of the lineedits
#define LINEEDIT_WIDTH	60 // the width of the lineedits
#define DIRBUTTON_HEIGHT LINEEDIT_HEIGHT // the height of the direction-buttons
#define DIRBUTTON_WIDTH  25 // the width of the direction-buttons
#define STDBUTTON_HEIGHT 25 // the height of the standard-buttons (OK and cancel)
#define STDBUTTON_WIDTH  70 // the width of the standard-buttons
#define DISTANCE 4 // the distance between the controls	(should be even)


CConfigDialog::CConfigDialog(int nKeyNW, int nKeyNE, int nKeySW, int nKeySE, QWidget * pParent, const char * pszName)
	: QDialog(pParent, pszName, true),
	  m_fWaitingForKey(false)
{

// added by Holger Priebs ( 18 Nov 99 )

	setCaption("Configure Keys");

	configpix = new QPixmap( configback_xpm );
	button_nw = new QPixmap( nw_xpm );
	button_ne = new QPixmap( ne_xpm );
	button_sw = new QPixmap( sw_xpm );
	button_se = new QPixmap( se_xpm );

	QColorGroup normal( QColor( QRgb(0) ), QColor( QRgb(16296080) ), QColor( QRgb(16310472) ), QColor( QRgb(8412240) ), QColor( QRgb(16287864) ), QColor( QRgb(0) ), QColor( QRgb(16777215) ) );
	QColorGroup disabled( QColor( QRgb(0) ), QColor( QRgb(16769987) ), QColor( QRgb(16310472) ), QColor( QRgb(8412240) ), QColor( QRgb(16287864) ), QColor( QRgb(8421504) ), QColor( QRgb(16777215) ) );
	QColorGroup active( QColor( QRgb(0) ), QColor( QRgb(12632256) ), QColor( QRgb(16777215) ), QColor( QRgb(8421504) ), QColor( QRgb(10789024) ), QColor( QRgb(0) ), QColor( QRgb(16777215) ) );

	QPalette palette( normal, disabled, active );
// --------

	m_nActKeys[0] = nKeyNW;
	m_nActKeys[1] = nKeyNE;
	m_nActKeys[2] = nKeySW;
	m_nActKeys[3] = nKeySE;

	// resize the dialog
	setFixedSize( DISTANCE + LINEEDIT_WIDTH + DISTANCE + DIRBUTTON_WIDTH + DISTANCE + DIRBUTTON_WIDTH + DISTANCE + LINEEDIT_WIDTH + DISTANCE,
	              DISTANCE + LINEEDIT_HEIGHT + DISTANCE + LINEEDIT_HEIGHT + DISTANCE + STDBUTTON_HEIGHT + DISTANCE );

	// - initialize the editlines
	for (int i=0; i<4; i++)
	{	//m_pLineEdits[i]->setPalette( palette ); // Absturz!!
		m_pLineEdits[i] = new QLineEdit(this);
		m_pLineEdits[i]->setEnabled(false);
		m_pLineEdits[i]->setText(getKeycodeName(m_nActKeys[i]));
		m_pLineEdits[i]->show();

	}

// added by Holger Priebs (16 Nov 99) ----

	m_pLineEdits[0]->setPalette( palette );
	m_pLineEdits[1]->setPalette( palette );
	m_pLineEdits[2]->setPalette( palette );
	m_pLineEdits[3]->setPalette( palette );
// ---------------------------------------

	m_pLineEdits[0]->setGeometry( DISTANCE, DISTANCE, LINEEDIT_WIDTH, LINEEDIT_HEIGHT );
	m_pLineEdits[1]->setGeometry( 4*DISTANCE + LINEEDIT_WIDTH + 2*DIRBUTTON_WIDTH, DISTANCE, LINEEDIT_WIDTH, LINEEDIT_HEIGHT );
	m_pLineEdits[2]->setGeometry( DISTANCE, 2*DISTANCE + LINEEDIT_HEIGHT, LINEEDIT_WIDTH, LINEEDIT_HEIGHT );
	m_pLineEdits[3]->setGeometry( 4*DISTANCE + LINEEDIT_WIDTH + 2*DIRBUTTON_WIDTH, 2*DISTANCE + LINEEDIT_HEIGHT, LINEEDIT_WIDTH, LINEEDIT_HEIGHT );

	// - initialize the direction-buttons
	for (int i=0; i<4; i++)	
	{
		m_pDirButtons[i] = new QPushButton(this);
		m_pDirButtons[i]->show();
	}

// removed by Holger Priebs ( 18 Nov 99 )
	
	//m_pDirButtons[0]->setText("\\");
	//m_pDirButtons[1]->setText("/");
	//m_pDirButtons[2]->setText("/");
	//m_pDirButtons[3]->setText("\\");

// -------------------------------

	m_pDirButtons[0]->setGeometry( 2*DISTANCE+LINEEDIT_WIDTH, DISTANCE, DIRBUTTON_WIDTH, DIRBUTTON_HEIGHT );
	m_pDirButtons[1]->setGeometry( 3*DISTANCE + LINEEDIT_WIDTH + DIRBUTTON_WIDTH, DISTANCE, DIRBUTTON_WIDTH, DIRBUTTON_HEIGHT  );
	m_pDirButtons[2]->setGeometry( 2*DISTANCE+LINEEDIT_WIDTH, 2*DISTANCE + LINEEDIT_HEIGHT, DIRBUTTON_WIDTH, DIRBUTTON_HEIGHT  );
	m_pDirButtons[3]->setGeometry( 3*DISTANCE + LINEEDIT_WIDTH + DIRBUTTON_WIDTH, 2*DISTANCE + LINEEDIT_HEIGHT, DIRBUTTON_WIDTH, DIRBUTTON_HEIGHT  );

// added by Holger Priebs (16 Nov 99) ----

	m_pDirButtons[0]->setPalette( palette );
	m_pDirButtons[1]->setPalette( palette );
	m_pDirButtons[2]->setPalette( palette );
	m_pDirButtons[3]->setPalette( palette );

// ---------------------------------------

	connect( m_pDirButtons[0], SIGNAL(clicked()), SLOT(pressedNWButton()) );
	connect( m_pDirButtons[1], SIGNAL(clicked()), SLOT(pressedNEButton()) );
	connect( m_pDirButtons[2], SIGNAL(clicked()), SLOT(pressedSWButton()) );
	connect( m_pDirButtons[3], SIGNAL(clicked()), SLOT(pressedSEButton()) );

// added by Holger Priebs ( 18 Nov 99 )

	m_pDirButtons[0]->setPixmap( *button_nw );
	m_pDirButtons[1]->setPixmap( *button_ne );
	m_pDirButtons[2]->setPixmap( *button_sw );
	m_pDirButtons[3]->setPixmap( *button_se );

// ------------------------------------

	// - initialize the standard-buttons
	QPushButton* pButtonOK;
	QPushButton* pButtonCancel;

	pButtonOK = new QPushButton("OK", this);
	pButtonCancel = new QPushButton("Cancel", this);

	pButtonOK->setGeometry( DISTANCE + LINEEDIT_WIDTH + DISTANCE + DIRBUTTON_WIDTH - STDBUTTON_WIDTH, DISTANCE + LINEEDIT_HEIGHT + DISTANCE + LINEEDIT_HEIGHT + DISTANCE, STDBUTTON_WIDTH, STDBUTTON_HEIGHT);
	pButtonCancel->setGeometry( DISTANCE + LINEEDIT_WIDTH + DISTANCE + DIRBUTTON_WIDTH + DISTANCE, DISTANCE + LINEEDIT_HEIGHT + DISTANCE + LINEEDIT_HEIGHT + DISTANCE, STDBUTTON_WIDTH, STDBUTTON_HEIGHT);

	pButtonOK->setPalette( palette );
	pButtonCancel->setPalette( palette );

	pButtonOK->show();
	pButtonCancel->show();

	connect( pButtonOK, SIGNAL(clicked()), SLOT(pressedOKButton()) );
	connect( pButtonCancel, SIGNAL(clicked()), SLOT(pressedCancelButton()) );


}

void CConfigDialog::paintEvent( QPaintEvent * )
{
	bitBlt(this, 0, 0, configpix );
}

void CConfigDialog::pressedDirButton(int nDir)
{
	// nDir: 0 - NW; 1 - NE; 2 - SW; 3 - SE

	m_fWaitingForKey = true;
	m_nActDirection = nDir;

	for (int i=0; i<4; i++)
		m_pDirButtons[i]->setDown(false);

	m_pDirButtons[nDir]->setDown(true);
}

void CConfigDialog::pressedOKButton()
{
	m_fWaitingForKey = false;
	emit newKeys(m_nActKeys[0], m_nActKeys[1], m_nActKeys[2], m_nActKeys[3]);
	done(1);
}

void CConfigDialog::pressedCancelButton()
{
	m_fWaitingForKey = false;
	done(0);
}

void CConfigDialog::keyPressEvent( QKeyEvent* e)
{
	if (m_fWaitingForKey)
	{
		int nKey = e->key();
		if ( ((nKey>=Key_Home)&&(nKey<=Key_PageDown)) ||
		     ((nKey>=Key_Space)&&(nKey<=Key_AsciiTilde)) )
		{
			//checking for double occurence:
			bool fFirstOccurence = true;
			for (int i=0; i<4; i++)
				if ((nKey == m_nActKeys[i])&&(i != m_nActDirection))
					fFirstOccurence = false;
			
			if (fFirstOccurence)
			{
				// accept the new key			
				m_fWaitingForKey = false;
				m_nActKeys[m_nActDirection] = nKey;
				m_pLineEdits[m_nActDirection]->setText(getKeycodeName(nKey));
				m_pDirButtons[m_nActDirection]->setDown(false);
			}
		}
	}
}


// non-member function !!!
QString getKeycodeName(int nKeycode)
{
	switch(nKeycode)
	{
		case Key_Home: return "Home"; break;
		case Key_End: return "End"; break;
		case Key_Left: return "Left"; break;
		case Key_Up: return "Up"; break;
		case Key_Right: return "Right"; break;
		case Key_Down: return "Down"; break;
		case Key_PageUp: return "PageUp"; break;
		case Key_PageDown: return "PageDown"; break;

		case Key_Space: return "Space"; break;
		case Key_Exclam: return "!"; break;
		case Key_QuoteDbl: return "\""; break;
		case Key_NumberSign: return "#"; break;
		case Key_Dollar: return "$"; break;
		case Key_Percent: return "%"; break;
		case Key_Ampersand: return "&"; break;
		case Key_Apostrophe: return "'"; break;
		case Key_ParenLeft: return "("; break;
		case Key_ParenRight: return ")"; break;
		case Key_Asterisk: return "*"; break;
		case Key_Plus: return "+"; break;
		case Key_Comma: return ","; break;
		case Key_Minus: return "-"; break;
		case Key_Period: return "."; break;
		case Key_Slash: return "/"; break;
		case Key_0: return "0"; break;
		case Key_1: return "1"; break;
		case Key_2: return "2"; break;
		case Key_3: return "3"; break;
		case Key_4: return "4"; break;
		case Key_5: return "5"; break;
		case Key_6: return "6"; break;
		case Key_7: return "7"; break;
		case Key_8: return "8"; break;
		case Key_9: return "9"; break;
		case Key_Colon: return ","; break;
		case Key_Semicolon: return ";"; break;
		case Key_Less: return "<"; break;
		case Key_Equal: return "="; break;
		case Key_Greater: return ">"; break;
		case Key_Question: return "?"; break;
		case Key_At: return ""; break;
		case Key_A: return "a"; break;
		case Key_B: return "b"; break;
		case Key_C: return "c"; break;
		case Key_D: return "d"; break;
		case Key_E: return "e"; break;
		case Key_F: return "f"; break;
		case Key_G: return "g"; break;
		case Key_H: return "h"; break;
		case Key_I: return "i"; break;
		case Key_J: return "j"; break;
		case Key_K: return "k"; break;
		case Key_L: return "l"; break;
		case Key_M: return "m"; break;
		case Key_N: return "n"; break;
		case Key_O: return "o"; break;
		case Key_P: return "p"; break;
		case Key_Q: return "q"; break;
		case Key_R: return "r"; break;
		case Key_S: return "s"; break;
		case Key_T: return "t"; break;
		case Key_U: return "u"; break;
		case Key_V: return "v"; break;
		case Key_W: return "w"; break;
		case Key_X: return "x"; break;
		case Key_Y: return "y"; break;
		case Key_Z: return "z"; break;
		case Key_BracketLeft: return "["; break;
		case Key_Backslash: return "\\"; break;
		case Key_BracketRight: return "]"; break;
		case Key_AsciiCircum: return "Circum"; break;
		case Key_Underscore: return "_"; break;
		case Key_QuoteLeft: return "`"; break;
		case Key_BraceLeft: return "{"; break;
		case Key_Bar: return "|"; break;
		case Key_BraceRight: return "}"; break;
		case Key_AsciiTilde: return "~"; break;
		default: return "unknown";
	}
	return "w";		
}
